==> openGion6.5.0.0_src.zip: gf/src/org/opengion/fukurou/util/SystemParameter.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.util;

import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

import static org.opengion.fukurou.system.HybsConst.CR;				// 6.1.0.0 (2014/12/26) refactoring
import static org.opengion.fukurou.system.HybsConst.BUFFER_MIDDLE;	// 6.1.0.0 (2014/12/26) refactoring
import org.opengion.fukurou.system.OgRuntimeException ;		// 6.4.2.0 (2016/01/29)

/**
 * SystemParameter は、{&#064;XXXX} 文字列を処理するクラスです。
 * このクラスでは、{&#064;XXXX} 文字列を別の文字列と置き換えることや、
 * 予め予約されている予約語 {&#064;DATE.XXXX} 文字列を置き換えます。
 * 通常の {&#064;XXXX} 文字列の置き換えは、キーと値のペアを、HybsEntry オブジェクトに
 * セットして、その配列を受け取って処理します。
 *
 * 以下の値はあらかじめ、動的に作成されます。
 * ・DATE.YMD       ８byte の今日のシステム日付(yyyyMMdd)
 * ・DATE.YMDH    １４byte の今日のシステム日時(yyyyMMddHHmmss)
 * ・DATE.HMS       ６byte の今日のシステム時間(HHmmss)
 *
 * @og.group ユーティリティ
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public final class SystemParameter {

	private final String	original ;

	private final String[] clms;
	private final String[] formats;

	/**
	 *  {&#064;XXXX} の特殊文字を含む文字列を、置き換えます。
	 * 対象外の文字列は、そのまま、残されます。
	 *
	 * @og.rev 5.1.8.0 (2010/07/01) パース方法見直し(StringTokenizerでは、{&#064;XXXX}が連続してある場合に対応できない)
	 * @og.rev 5.3.2.0 (2011/02/01) original データを、パース結果を利用するように修正する。
	 * @og.rev 5.3.4.0 (2011/04/01) {&#064;DATE.XXXX} を処理できるように機能追加
	 * @og.rev 5.3.5.0 (2011/05/01) {&#064;SYS.XXXX} は、廃止
	 * @og.rev 5.5.7.2 (2012/10/09) rightNow をCalendarオブジェクト ではなく、String時刻とします。
	 * @og.rev 5.5.7.2 (2012/10/09) HybsDateUtil を利用するように修正します。
	 *
	 * @param	orig	変換する文字列(オリジナル)
	 */
	public SystemParameter( final String orig ) {
		if( orig == null || orig.isEmpty() || orig.indexOf( "{@" ) < 0 ) {
			clms     = null;
			formats  = null;
			original = orig;		// 5.3.2.0 (2011/02/01)
		}
		else {
			final StringBuilder buf = new StringBuilder(orig.length());		// 5.3.2.0 (2011/02/01)

			final ArrayList<String> fmtList = new ArrayList<>();
			final ArrayList<String> clmList = new ArrayList<>();

			// 5.1.8.0 (2010/07/01) パース方法見直し
			int start = 0;
			int index = orig.indexOf( "{@" );
			String val ;
			while( index >= 0 ) {
				val = orig.substring( start, index );					// 5.3.4.0 (2011/04/01)
				buf.append(  val );
				fmtList.add( val );
				final int end = orig.indexOf( '}',index );
				if( end < 0 ) {
					final String errMsg = "{@ と } との対応関係がずれています。" + CR
								+ "str=[" + orig + "] : index=" + index ;
					throw new OgRuntimeException( errMsg );
				}
				final String param = orig.substring( index+2,end );
				if( param.startsWith( "DATE." ) ) {		// 5.3.5.0 (2011/05/01) {&#064;SYS.XXXX} は、廃止
					val = getDateFormat( param.substring( 5 ) );	// 5.5.7.2 (2012/10/09) HybsDateUtil を利用時に "DATE." は不要
					clmList.add( null );		// パースした場合は、clmList は、使用しない。
					buf.append( val );
				}
				else {
					clmList.add( param );
					buf.append( "{@" ).append( param ).append( '}' );		// 元のままの文字列を生成		// 6.0.2.5 (2014/10/31) char を append する。
				}
				start = end+1;
				index = orig.indexOf( "{@",start );
			}
			val = orig.substring( start, orig.length() );				// 5.3.4.0 (2011/04/01)
			buf.append(  val );
			fmtList.add( val );

			original = buf.toString();		// 5.3.2.0 (2011/02/01)
			if( original.indexOf( "{@" ) < 0 ) {
				clms     = null;
				formats  = null;
			}
			else {
				clms	= clmList.toArray( new String[clmList.size()] );
				formats = fmtList.toArray( new String[fmtList.size()] );
			}
		}
	}

	/**
	 * 日付関係の情報を簡易的に取り出す処理を行います。
	 *
	 * 引数は、"KEY AA(第1引数) BB(第2引数) CC(第3引数)" という状態で受け取ります。
	 * 引数は、なくてもかまいません。ただし、引数にリクエストパラメータ(@で始まる引数)は使えません。
	 *
	 * 処理の詳細は、{@link org.opengion.fukurou.util.HybsDateUtil#getDateFormat( String,String,String,int ) }
	 * または、{@link org.opengion.hayabusa.taglib.CommonTagSupport#getDateFormat( String ) } を
	 * 参照してください。
	 *
	 * @og.rev 5.3.4.0 (2011/04/01) 新規追加
	 * @og.rev 5.5.7.2 (2012/10/09) HybsDateUtil を利用するように修正します。
	 * @og.rev 5.5.8.2 (2012/11/09) prmA の判定に、null と ゼロ文字列を判定する。
	 * @og.rev 5.7.4.1 (2014/03/14) AA 引数の@解析後のコマンド判定方法を、８ケタ以下から先頭が数字以外に変更します。
	 * @og.rev 5.7.4.1 (2014/03/14) taglib.CommonTagSupport#getDateFormat( String ) と処理を合わせます。
	 * @og.rev 5.7.4.2 (2014/03/20) リクエストパラメータ(@で始まる引数)は使えません。
	 * @og.rev 6.4.1.1 (2016/01/16) StringUtil#csv2Array(String,char) → StringUtil#csv2Array(String,char,int) で、配列長を固定化する。
	 *
	 * @param   value パラメータ(引数は、"DATE.KEY AA BB" などという状態)
	 *
	 * @return   メッセージ情報
	 * @og.rtnNotNull
	 * @see		org.opengion.fukurou.util.HybsDateUtil#getDateFormat( String,String,String,int )
	 * @see		org.opengion.hayabusa.taglib.CommonTagSupport#getDateFormat( String )
	 */
	private String getDateFormat( final String value ) {
		// 6.4.1.1 (2016/01/16) StringUtil#csv2Array(String,char) → StringUtil#csv2Array(String,char,int) で、配列長を固定化する。
		// 足りない分は、null ではなく、"" がセットされてくる。
		final String[] vals = StringUtil.csv2Array( value,' ',4 );

		// 5.7.4.2 (2014/03/20) 現時点では、SystemParameter 処理にはリクエスト変数は使えないので、@変数も使えない。
		// vals[0] は、key なので、vals[1] から回す。
		for( int i=1; i<vals.length; i++ ) {
			if( !vals[i].isEmpty() && vals[i].charAt(0) == '@' ) {
				final String errMsg = "第" + i + "引数に、リクエストパラメータ(@で始まる引数)は使えません。value=[" + value + "]" ;
				throw new OgRuntimeException( errMsg );
			}
		}

		// 5.7.4.1 (2014/03/14) AA 引数の@解析後のコマンド判定方法を、８ケタ以下から先頭が数字以外に変更します。
		if( !vals[1].isEmpty() ) {
			final char chA = vals[1].charAt(0);
			if( chA < '0' || chA > '9' ) {		// 先頭が、数字以外の場合は、コマンドなので、一つずつずらす。
				vals[3] = vals[2];
				vals[2] = vals[1];
				vals[1] = null;
			}
		}

		// 5.7.4.1 (2014/03/14) CC(第3引数)を、"H" , "D" , "M" 以外でも使用できるように拡張します。
		int intC = 0;
		if( !vals[3].isEmpty() ) {
			try { 
				intC = Integer.parseInt( vals[3] );
			}
			catch( NumberFormatException ex ) {
				final String errMsg = "第3引数が数字ではありません。value=[" + value + "]" 
								+ ex.getMessage() ;
				System.err.println( errMsg );
			}
		}

		// AA(第1引数) が null か、isEmpty() の場合は、現在時刻が使用される。
		return HybsDateUtil.getDateFormat( vals[0],vals[1],vals[2],intC );	// 5.7.4.1 (2014/03/14) CC第3引数を拡張します。

//		// 5.7.4.1 (2014/03/14) taglib.CommonTagSupport#getDateFormat( String ) と処理を合わせます。
//		final String[] vals = StringUtil.csv2Array( value,' ' );
//
//		final String key = vals[0] ;
//
//		String prmA = vals.length >= 2 ? vals[1] : null ;
//		String prmB = vals.length >= 3 ? vals[2] : null ;
//		String prmC = vals.length >= 4 ? vals[vals.length-1] : null ;		// 互換性。最後の値が、CC引数
//
//		// 5.7.4.2 (2014/03/20) 現時点では、SystemParameter 処理にはリクエスト変数は使えないので、@変数も使えない。
//		if( prmA != null && prmA.length() > 0 && prmA.charAt(0) == '@' ) {	// 6.1.0.0 (2014/12/26) refactoring
//			final String errMsg = "AA引数に、リクエストパラメータ(@で始まる引数)は使えません。value=[" + value + "]" ;
//			throw new OgRuntimeException( errMsg );
//		}
//
//		if( prmB != null && prmB.length() > 0 && prmB.charAt(0) == '@' ) {	// 6.1.0.0 (2014/12/26) refactoring
//			final String errMsg = "BB引数に、リクエストパラメータ(@で始まる引数)は使えません。value=[" + value + "]" ;
//			throw new OgRuntimeException( errMsg );
//		}
//
//		if( prmC != null && prmC.length() > 0 && prmC.charAt(0) == '@' ) {	// 6.1.0.0 (2014/12/26) refactoring
//			final String errMsg = "CC引数に、リクエストパラメータ(@で始まる引数)は使えません。value=[" + value + "]" ;
//			throw new OgRuntimeException( errMsg );
//		}
//
//		// 5.7.4.1 (2014/03/14) AA 引数の@解析後のコマンド判定方法を、８ケタ以下から先頭が数字以外に変更します。
//		if( prmA != null && prmA.length() > 0 ) {
//			final char chA = prmA.charAt(0);
//			if( chA < '0' || chA > '9' ) {		// 先頭が、数字以外の場合は、コマンドなので、一つずつずらす。
//				prmC = prmB;
//				prmB = prmA;
//				prmA = null;
//			}
//		}
//
//		// 5.7.4.1 (2014/03/14) CC 引数を、"H" , "D" , "M" 以外でも使用できるように拡張します。
//		int intC = 0;
//		if( prmC != null && prmC.length() > 0 ) {
//			try { 
//				intC = Integer.parseInt( prmC );
//			}
//			catch( NumberFormatException ex ) {
//				final String errMsg = "CC引数が数字ではありません。value=[" + value + "]" 
//								+ ex.getMessage() ;
//				System.err.println( errMsg );
//			}
//		}
//
//		// prmA が null か、isEmpty() の場合は、現在時刻が使用される。
//		return HybsDateUtil.getDateFormat( key,prmA,prmB,intC );	// 5.7.4.1 (2014/03/14) CC 引数を拡張します。
	}

	/**
	 *  {&#064;XXXX} の特殊文字を含む文字列を、置き換えます。
	 * 対象外の文字列は、そのまま、残されます。
	 *
	 * @og.rev 5.3.4.0 (2011/04/01) 判定方法 修正
	 *
	 * @param	entry	置換文字列のキーと値のペアを管理しているEntryオブジェクトの配列(可変長引数)
	 *
	 * @return	置換後の文字列
	 */
	public String replace( final HybsEntry... entry ) {
		// 6.1.1.0 (2015/01/17) 可変長引数でもnullは来る。
		if( formats == null || entry == null || entry.length == 0 ) { return original; }

		// HybsEntry[] データを、Mapにマッピングします。
		final Map<String, String> sysMap = new HashMap<>();
		final int size = entry.length;
		for( int i=0; i<size; i++ ) {
			sysMap.put( entry[i].getKey(),entry[i].getValue() );
		}

		return replace( sysMap );
	}

	/**
	 *  {&#064;XXXX} の特殊文字を含む文字列を、置き換えます。
	 * 対象外の文字列は、そのまま、残されます。
	 *
	 * @param  map  置換文字列のキーと値のペアを管理しているMapオブジェクト
	 *
	 * @return	置換後の文字列
	 */
	public String replace( final Map<String,String> map ) {
		if( formats == null ) { return original; }		// 5.3.4.0 (2011/04/01) 判定方法 修正
		if( map == null || map.isEmpty() ) { return original; }

		final StringBuilder sb = new StringBuilder( BUFFER_MIDDLE );
		for( int i=0; i<formats.length; i++ ) {
			sb.append( formats[i] );
			if( i < clms.length && clms[i] != null ) {		// 5.3.4.0 (2011/04/01) nullチェック追加
				sb.append( StringUtil.nval( map.get( clms[i] ), "" ) );
			}
		}

		return sb.toString();
	}

	/**
	 * フォーマットをパースした結果から、カラム一覧を配列形式で返します。
	 *
	 * @og.rev 5.1.7.0 (2010/06/01) 新規作成
	 *
	 * @return カラム配列
	 * @og.rtnNotNull
	 */
	public String[] getColumns() {
		// 6.4.1.1 (2016/01/16) PMD refactoring. A method should have only one exit point, and that should be the last statement in the method
		return clms == null ? new String[0] : clms.clone();

//		if( clms == null ) { return new String[0]; }
//		return clms.clone();
	}

	/**
	 * フォーマットをパースした結果から、フォーマット一覧を配列形式で返します。
	 *
	 * @og.rev 5.1.7.0 (2010/06/01) 新規作成
	 *
	 * @return フォーマット配列
	 * @og.rtnNotNull
	 */
	public String[] getFormats() {
		// 6.4.1.1 (2016/01/16) PMD refactoring. A method should have only one exit point, and that should be the last statement in the method
		return formats == null ? new String[0] : formats.clone();

//		if( formats == null ) { return new String[0]; }
//		return formats.clone();
	}
}

==> openGion6.7.4.1_src.zip: gf/src/org/opengion/fukurou/util/SystemParameter.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.util;

import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

import static org.opengion.fukurou.system.HybsConst.CR;				// 6.1.0.0 (2014/12/26) refactoring
import static org.opengion.fukurou.system.HybsConst.BUFFER_MIDDLE;	// 6.1.0.0 (2014/12/26) refactoring
import org.opengion.fukurou.system.OgRuntimeException ;		// 6.4.2.0 (2016/01/29)

/**
 * SystemParameter は、{&#064;XXXX} 文字列を処理するクラスです。
 * このクラスでは、{&#064;XXXX} 文字列を別の文字列と置き換えることや、
 * 予め予約されている予約語 {&#064;DATE.XXXX} 文字列を置き換えます。
 * 通常の {&#064;XXXX} 文字列の置き換えは、キーと値のペアを、HybsEntry オブジェクトに
 * セットして、その配列を受け取って処理します。
 *
 * 以下の値はあらかじめ、動的に作成されます。
 * ・DATE.YMD       ８byte の今日のシステム日付(yyyyMMdd)
 * ・DATE.YMDH    １４byte の今日のシステム日時(yyyyMMddHHmmss)
 * ・DATE.HMS       ６byte の今日のシステム時間(HHmmss)
 *
 * @og.group ユーティリティ
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public final class SystemParameter {

	private final String	original ;

	private final String[] clms;
	private final String[] formats;

	/**
	 *  {&#064;XXXX} の特殊文字を含む文字列を、置き換えます。
	 * 対象外の文字列は、そのまま、残されます。
	 *
	 * @og.rev 5.1.8.0 (2010/07/01) パース方法見直し(StringTokenizerでは、{&#064;XXXX}が連続してある場合に対応できない)
	 * @og.rev 5.3.2.0 (2011/02/01) original データを、パース結果を利用するように修正する。
	 * @og.rev 5.3.4.0 (2011/04/01) {&#064;DATE.XXXX} を処理できるように機能追加
	 * @og.rev 5.3.5.0 (2011/05/01) {&#064;SYS.XXXX} は、廃止
	 * @og.rev 5.5.7.2 (2012/10/09) rightNow をCalendarオブジェクト ではなく、String時刻とします。
	 * @og.rev 5.5.7.2 (2012/10/09) HybsDateUtil を利用するように修正します。
	 *
	 * @param	orig	変換する文字列(オリジナル)
	 */
	public SystemParameter( final String orig ) {
		if( orig == null || orig.isEmpty() || orig.indexOf( "{@" ) < 0 ) {
			clms     = null;
			formats  = null;
			original = orig;		// 5.3.2.0 (2011/02/01)
		}
		else {
			final StringBuilder buf = new StringBuilder(orig.length());		// 5.3.2.0 (2011/02/01)

			final ArrayList<String> fmtList = new ArrayList<>();
			final ArrayList<String> clmList = new ArrayList<>();

			// 5.1.8.0 (2010/07/01) パース方法見直し
			int start = 0;
			int index = orig.indexOf( "{@" );
			String val ;
			while( index >= 0 ) {
				val = orig.substring( start, index );					// 5.3.4.0 (2011/04/01)
				buf.append(  val );
				fmtList.add( val );
				final int end = orig.indexOf( '}',index );
				if( end < 0 ) {
					final String errMsg = "{@ と } との対応関係がずれています。" + CR
								+ "str=[" + orig + "] : index=" + index ;
					throw new OgRuntimeException( errMsg );
				}
				final String param = orig.substring( index+2,end );
				if( param.startsWith( "DATE." ) ) {		// 5.3.5.0 (2011/05/01) {&#064;SYS.XXXX} は、廃止
					val = getDateFormat( param.substring( 5 ) );	// 5.5.7.2 (2012/10/09) HybsDateUtil を利用時に "DATE." は不要
					clmList.add( null );		// パースした場合は、clmList は、使用しない。
					buf.append( val );
				}
				else {
					clmList.add( param );
					buf.append( "{@" ).append( param ).append( '}' );		// 元のままの文字列を生成		// 6.0.2.5 (2014/10/31) char を append する。
				}
				start = end+1;
				index = orig.indexOf( "{@",start );
			}
			val = orig.substring( start, orig.length() );				// 5.3.4.0 (2011/04/01)
			buf.append(  val );
			fmtList.add( val );

			original = buf.toString();		// 5.3.2.0 (2011/02/01)
			if( original.indexOf( "{@" ) < 0 ) {
				clms     = null;
				formats  = null;
			}
			else {
				clms	= clmList.toArray( new String[clmList.size()] );
				formats = fmtList.toArray( new String[fmtList.size()] );
			}
		}
	}

	/**
	 * 日付関係の情報を簡易的に取り出す処理を行います。
	 *
	 * 引数は、"KEY AA(第1引数) BB(第2引数) CC(第3引数)" という状態で受け取ります。
	 * 引数は、なくてもかまいません。ただし、引数にリクエストパラメータ(@で始まる引数)は使えません。
	 *
	 * 処理の詳細は、{@link org.opengion.fukurou.util.HybsDateUtil#getDateFormat( String,String,String,int ) }
	 * または、{@link org.opengion.hayabusa.taglib.CommonTagSupport#getDateFormat( String ) } を
	 * 参照してください。
	 *
	 * @og.rev 5.3.4.0 (2011/04/01) 新規追加
	 * @og.rev 5.5.7.2 (2012/10/09) HybsDateUtil を利用するように修正します。
	 * @og.rev 5.5.8.2 (2012/11/09) prmA の判定に、null と ゼロ文字列を判定する。
	 * @og.rev 5.7.4.1 (2014/03/14) AA 引数の@解析後のコマンド判定方法を、８ケタ以下から先頭が数字以外に変更します。
	 * @og.rev 5.7.4.1 (2014/03/14) taglib.CommonTagSupport#getDateFormat( String ) と処理を合わせます。
	 * @og.rev 5.7.4.2 (2014/03/20) リクエストパラメータ(@で始まる引数)は使えません。
	 * @og.rev 6.4.1.1 (2016/01/16) StringUtil#csv2Array(String,char) → StringUtil#csv2Array(String,char,int) で、配列長を固定化する。
	 *
	 * @param   value パラメータ(引数は、"DATE.KEY AA BB" などという状態)
	 *
	 * @return   メッセージ情報
	 * @og.rtnNotNull
	 * @see		org.opengion.fukurou.util.HybsDateUtil#getDateFormat( String,String,String,int )
	 * @see		org.opengion.hayabusa.taglib.CommonTagSupport#getDateFormat( String )
	 */
	private String getDateFormat( final String value ) {
		// 6.4.1.1 (2016/01/16) StringUtil#csv2Array(String,char) → StringUtil#csv2Array(String,char,int) で、配列長を固定化する。
		// 足りない分は、null ではなく、"" がセットされてくる。
		final String[] vals = StringUtil.csv2Array( value,' ',4 );

		// 5.7.4.2 (2014/03/20) 現時点では、SystemParameter 処理にはリクエスト変数は使えないので、@変数も使えない。
		// vals[0] は、key なので、vals[1] から回す。
		for( int i=1; i<vals.length; i++ ) {
			if( !vals[i].isEmpty() && vals[i].charAt(0) == '@' ) {
				final String errMsg = "第" + i + "引数に、リクエストパラメータ(@で始まる引数)は使えません。value=[" + value + "]" ;
				throw new OgRuntimeException( errMsg );
			}
		}

		// 5.7.4.1 (2014/03/14) AA 引数の@解析後のコマンド判定方法を、８ケタ以下から先頭が数字以外に変更します。
		if( !vals[1].isEmpty() ) {
			final char chA = vals[1].charAt(0);
			if( chA < '0' || chA > '9' ) {		// 先頭が、数字以外の場合は、コマンドなので、一つずつずらす。
				vals[3] = vals[2];
				vals[2] = vals[1];
				vals[1] = null;
			}
		}

		// 5.7.4.1 (2014/03/14) CC(第3引数)を、"H" , "D" , "M" 以外でも使用できるように拡張します。
		int intC = 0;
		if( !vals[3].isEmpty() ) {
			try { 
				intC = Integer.parseInt( vals[3] );
			}
			catch( final NumberFormatException ex ) {
				final String errMsg = "第3引数が数字ではありません。value=[" + value + "]" 
								+ ex.getMessage() ;
				System.err.println( errMsg );
			}
		}

		// AA(第1引数) が null か、isEmpty() の場合は、現在時刻が使用される。
		return HybsDateUtil.getDateFormat( vals[0],vals[1],vals[2],intC );	// 5.7.4.1 (2014/03/14) CC第3引数を拡張します。

//		// 5.7.4.1 (2014/03/14) taglib.CommonTagSupport#getDateFormat( String ) と処理を合わせます。
//		final String[] vals = StringUtil.csv2Array( value,' ' );
//
//		final String key = vals[0] ;
//
//		String prmA = vals.length >= 2 ? vals[1] : null ;
//		String prmB = vals.length >= 3 ? vals[2] : null ;
//		String prmC = vals.length >= 4 ? vals[vals.length-1] : null ;		// 互換性。最後の値が、CC引数
//
//		// 5.7.4.2 (2014/03/20) 現時点では、SystemParameter 処理にはリクエスト変数は使えないので、@変数も使えない。
//		if( prmA != null && prmA.length() > 0 && prmA.charAt(0) == '@' ) {	// 6.1.0.0 (2014/12/26) refactoring
//			final String errMsg = "AA引数に、リクエストパラメータ(@で始まる引数)は使えません。value=[" + value + "]" ;
//			throw new OgRuntimeException( errMsg );
//		}
//
//		if( prmB != null && prmB.length() > 0 && prmB.charAt(0) == '@' ) {	// 6.1.0.0 (2014/12/26) refactoring
//			final String errMsg = "BB引数に、リクエストパラメータ(@で始まる引数)は使えません。value=[" + value + "]" ;
//			throw new OgRuntimeException( errMsg );
//		}
//
//		if( prmC != null && prmC.length() > 0 && prmC.charAt(0) == '@' ) {	// 6.1.0.0 (2014/12/26) refactoring
//			final String errMsg = "CC引数に、リクエストパラメータ(@で始まる引数)は使えません。value=[" + value + "]" ;
//			throw new OgRuntimeException( errMsg );
//		}
//
//		// 5.7.4.1 (2014/03/14) AA 引数の@解析後のコマンド判定方法を、８ケタ以下から先頭が数字以外に変更します。
//		if( prmA != null && prmA.length() > 0 ) {
//			final char chA = prmA.charAt(0);
//			if( chA < '0' || chA > '9' ) {		// 先頭が、数字以外の場合は、コマンドなので、一つずつずらす。
//				prmC = prmB;
//				prmB = prmA;
//				prmA = null;
//			}
//		}
//
//		// 5.7.4.1 (2014/03/14) CC 引数を、"H" , "D" , "M" 以外でも使用できるように拡張します。
//		int intC = 0;
//		if( prmC != null && prmC.length() > 0 ) {
//			try { 
//				intC = Integer.parseInt( prmC );
//			}
//			catch( final NumberFormatException ex ) {
//				final String errMsg = "CC引数が数字ではありません。value=[" + value + "]" 
//								+ ex.getMessage() ;
//				System.err.println( errMsg );
//			}
//		}
//
//		// prmA が null か、isEmpty() の場合は、現在時刻が使用される。
//		return HybsDateUtil.getDateFormat( key,prmA,prmB,intC );	// 5.7.4.1 (2014/03/14) CC 引数を拡張します。
	}

	/**
	 *  {&#064;XXXX} の特殊文字を含む文字列を、置き換えます。
	 * 対象外の文字列は、そのまま、残されます。
	 *
	 * @og.rev 5.3.4.0 (2011/04/01) 判定方法 修正
	 *
	 * @param	entry	置換文字列のキーと値のペアを管理しているEntryオブジェクトの配列(可変長引数)
	 *
	 * @return	置換後の文字列
	 */
	public String replace( final HybsEntry... entry ) {
		// 6.1.1.0 (2015/01/17) 可変長引数でもnullは来る。
		if( formats == null || entry == null || entry.length == 0 ) { return original; }

		// HybsEntry[] データを、Mapにマッピングします。
		final Map<String, String> sysMap = new HashMap<>();
		final int size = entry.length;
		for( int i=0; i<size; i++ ) {
			sysMap.put( entry[i].getKey(),entry[i].getValue() );
		}

		return replace( sysMap );
	}

	/**
	 *  {&#064;XXXX} の特殊文字を含む文字列を、置き換えます。
	 * 対象外の文字列は、そのまま、残されます。
	 *
	 * @param  map  置換文字列のキーと値のペアを管理しているMapオブジェクト
	 *
	 * @return	置換後の文字列
	 */
	public String replace( final Map<String,String> map ) {
		if( formats == null ) { return original; }		// 5.3.4.0 (2011/04/01) 判定方法 修正
		if( map == null || map.isEmpty() ) { return original; }

		final StringBuilder sb = new StringBuilder( BUFFER_MIDDLE );
		for( int i=0; i<formats.length; i++ ) {
			sb.append( formats[i] );
			if( i < clms.length && clms[i] != null ) {		// 5.3.4.0 (2011/04/01) nullチェック追加
				sb.append( StringUtil.nval( map.get( clms[i] ), "" ) );
			}
		}

		return sb.toString();
	}

	/**
	 * フォーマットをパースした結果から、カラム一覧を配列形式で返します。
	 *
	 * @og.rev 5.1.7.0 (2010/06/01) 新規作成
	 *
	 * @return カラム配列
	 * @og.rtnNotNull
	 */
	public String[] getColumns() {
		// 6.4.1.1 (2016/01/16) PMD refactoring. A method should have only one exit point, and that should be the last statement in the method
		return clms == null ? new String[0] : clms.clone();

//		if( clms == null ) { return new String[0]; }
//		return clms.clone();
	}

	/**
	 * フォーマットをパースした結果から、フォーマット一覧を配列形式で返します。
	 *
	 * @og.rev 5.1.7.0 (2010/06/01) 新規作成
	 *
	 * @return フォーマット配列
	 * @og.rtnNotNull
	 */
	public String[] getFormats() {
		// 6.4.1.1 (2016/01/16) PMD refactoring. A method should have only one exit point, and that should be the last statement in the method
		return formats == null ? new String[0] : formats.clone();

//		if( formats == null ) { return new String[0]; }
//		return formats.clone();
	}
}
